##############################################################################
#
# upgradedb.tcl utility (upgrade databases 5.x -> 5.y, controller section must
# be handled separately: see vcutils/scripts/upgrade-database-v5)
#
# Implements the following algorithm:
#
#  - ensure that new.ini = template.ini
#
#  - for each section in new.ini:
#  
#     - case [configuration]:
#     - case [most recent]:
#
#         - for each entry in [configuration]:
#
#           - if the entry also exists in old.ini
#
#             - replace value in new.ini with value in old.ini
#
#           - if entry doesn't exist in old.ini, then leave
#             new.ini unchanged
#
#           - entries in old.ini that don't exist in
#             new.ini can be ignored
#
#     - case [base stations]:
#     - case [fields]:
#     - case [jobs]:
#     - case [implements]:
#     - case [tractors]:
#     - case [controller]:
#
#       - copy all entries from old.ini to new.ini
#
#     - case section exists in new.ini but not in old.ini
#
#       - leave new.ini unchanged
#
#     - case section exists in old.ini but not in new.ini
#
#       - ignore
#
# Copyright (c) 2003 IntegriNautics Corporation.
# All Rights Reserved. 
##############################################################################
if { [llength $argv] < 4} {
    puts "Usage: tclsh8.4 upgradeini.tcl {vcParentDir}\
            {workingDir} {old.ini} {new.ini}\n\
            e.g. tclsh8.4 upgradeini.tcl /mnt/apps/ \
            /configuration/currdir0/ oldDbmain.ini newDbmain.ini"
    exit
}
set vcParentDir [lindex $argv 0]
set workingDir [lindex $argv 1]
set oldIni [lindex $argv 2]
set newIni [lindex $argv 3]

puts "vcParentDir=$vcParentDir\nworkingDir=$workingDir\n\
        oldIni=$oldIni\nnewIni=$newIni"

source ${vcParentDir}vcutils/syncdisk/iniparse.tcl
source ${vcParentDir}vcutils/syncdisk/fsm.tcl

#-----------------------------------------------------------------------------
# helper procedures
#

if [catch {exec cp ${vcParentDir}vc/src/data/template.ini \
        ${workingDir}${newIni}}] {
    puts "Copying template.ini failed"
    exit -1                                                 ;# exit with error
} else {
    puts "Using ${vcParentDir}vc/src/data/template.ini as template"
}

# open and parse the old file
set oldId [iniparse:openfile ${workingDir}${oldIni}]
if {$oldId == -1} {
    puts "file ${workingDir}${oldIni} open failed"
    exit -1                                                 ;# exit with error
} else {
    puts "file ${workingDir}${oldIni} parsed successfully"
}

# open and parse the new file
set newId [iniparse:openfile ${workingDir}${newIni}]
if {$newId == -1} {
    puts "file ${workingDir}${newIni} open failed"
    exit -1                                                 ;# exit with error
} else {
    puts "file ${workingDir}${newIni} parsed successfully"
}

proc replaceFromOld {section} {
    global oldId newId
    puts "replacing section \[$section\] in new database..."
    set oldList [readini $oldId $section]
    set newList [readini $newId $section]
    foreach entry $newList {
        if {$entry == "!size"} continue  ;# leave !size as in the template.ini
            set matchIndex [lsearch -exact $oldList $entry]
        if {$matchIndex != -1} {                        ;# entry found in old?
            set oldData [readini $oldId $section $entry]
            writeini $newId $section $entry $oldData;# replace from old
            #puts "replaced $section:$entry in new"
        }
    }
}

proc copyFromOld {section} {
    global oldId newId
    puts "copying section \[$section\] from old database..."
    set oldList [readini $oldId $section]
    set newList [readini $newId $section]
    foreach entry $oldList {
        if {$entry == "!size"} continue  ;# leave !size as in the template.ini
        if {$entry == "SOFTWARE_VERSION"} continue  ;# leave data base software version as in the template.ini
        set oldData [readini $oldId $section $entry]
        writeini $newId $section $entry $oldData
        #puts "copied $section:$entry from old"
    }       
}


#-----------------------------------------------------------------------------
# process the sections 
# {MLE-030908: Currently this is a double point of maintenance}
#
replaceFromOld "configuration"
replaceFromOld "most recent"

copyFromOld "base stations"
copyFromOld "fields"
copyFromOld "jobs"
copyFromOld "implements"
copyFromOld "tractors"
copyFromOld "controller"
copyFromOld "icontrol"

#-----------------------------------------------------------------------------
# close .ini files and cleanup 
#
iniparse:closefile $oldId
iniparse:closefile $newId

puts "database upgrade successfull."


exit 0                                                    ;# exit with success
